@@ -7,6 +7,7 @@ import re
 import sys
 import time
 from datetime import timedelta
+from pathlib import Path
 
 from jade.enums import JobCompletionStatus, Status
 from jade.events import (
@@ -16,7 +17,6 @@ from jade.events import (
     EVENT_NAME_HPC_JOB_ASSIGNED,
     EVENT_NAME_HPC_JOB_STATE_CHANGE,
 )
-from jade.exceptions import ExecutionError
 from jade.hpc.common import HpcJobStatus, HpcType
 from jade.hpc.hpc_manager import HpcManager
 from jade.jobs.async_job_interface import AsyncJobInterface
@@ -37,6 +37,8 @@ logger = logging.getLogger(__name__)
 class HpcSubmitter:
     """Submits batches of jobs to HPC. Manages job ordering."""
 
+    LOCK_FILENAME = "submitter.lock"
+
     def __init__(self, config: JobConfiguration, config_file, cluster: Cluster, output):
         self._config = config
         self._submission_groups = make_submission_group_lookup(cluster.config.submission_groups)
@@ -115,34 +117,52 @@ class HpcSubmitter:
             poll_interval=self._poll_interval,
         )
         # Statuses may have changed since we last ran.
+        # Spurious exceptions (like network errors during an squeue call) here will cause this
+        # submitter to fail. Another submitter will try again later
+        # (unless this is the last submitter).
         queue.process_queue()
         completed_job_names, canceled_jobs = self._update_completed_jobs()
 
-        blocked_jobs = []
-        submitted_jobs = []
-        for group in self._cluster.config.submission_groups:
-            if not queue.is_full():
-                self._submit_batches(queue, group, blocked_jobs, submitted_jobs)
+        lock_file = Path(self._output) / self.LOCK_FILENAME
+        assert not lock_file.exists(), lock_file
+        lock_file.touch()
 
-        num_submissions = self._batch_index - starting_batch_index
-        logger.info(
-            "num_batches=%s num_submitted=%s num_blocked=%s new_completions=%s",
-            num_submissions,
-            len(submitted_jobs),
-            len(blocked_jobs),
-            len(completed_job_names),
-        )
+        # Start submitting jobs. If any unexpected exception prevents us from updating the
+        # status file, leave the lock_file in place and intentionally cause a deadlock.
+        try:
+            blocked_jobs = []
+            submitted_jobs = []
+            for group in self._cluster.config.submission_groups:
+                if not queue.is_full():
+                    self._submit_batches(queue, group, blocked_jobs, submitted_jobs)
 
-        hpc_job_ids = sorted([x.job_id for x in queue.outstanding_jobs])
-        self._update_status(
-            submitted_jobs,
-            blocked_jobs,
-            canceled_jobs,
-            hpc_job_ids,
-            completed_job_names,
-        )
+            num_submissions = self._batch_index - starting_batch_index
+            logger.info(
+                "num_batches=%s num_submitted=%s num_blocked=%s new_completions=%s",
+                num_submissions,
+                len(submitted_jobs),
+                len(blocked_jobs),
+                len(completed_job_names),
+            )
 
-        return self._is_complete()
+            hpc_job_ids = sorted([x.job_id for x in queue.outstanding_jobs])
+            self._update_status(
+                submitted_jobs,
+                blocked_jobs,
+                canceled_jobs,
+                hpc_job_ids,
+                completed_job_names,
+            )
+
+            is_complete = self._is_complete()
+            os.remove(lock_file)
+            return is_complete
+        except Exception:
+            logger.exception(
+                "An exception occurred while the submitter was active. "
+                "The state of the cluster is unknown. A deadlock will occur."
+            )
+            raise
 
     def _update_status(
         self, submitted_jobs, blocked_jobs, canceled_jobs, hpc_job_ids, completed_job_names
@@ -366,6 +386,7 @@ class AsyncHpcSubmitter(AsyncJobInterface):
         self._last_status = HpcJobStatus.NONE
         self._is_complete = False
         self._dry_run = dry_run
+        self._return_code = None
 
     def cancel(self):
         self._mgr.cancel_job(self._job_id)
@@ -397,6 +418,8 @@ class AsyncHpcSubmitter(AsyncJobInterface):
         return self._mgr
 
     def is_complete(self):
+        if self._is_complete:
+            return self._is_complete
         status = self._status_collector.check_status(self._job_id)
         if status != self._last_status:
             logger.info(
@@ -432,7 +455,7 @@ class AsyncHpcSubmitter(AsyncJobInterface):
     @property
     def return_code(self):
         assert self._is_complete
-        return 0
+        return self._return_code
 
     def run(self):
         assert self._submission_group is not None
@@ -444,7 +467,11 @@ class AsyncHpcSubmitter(AsyncJobInterface):
             dry_run=self._dry_run,
         )
         if result != Status.GOOD:
-            raise ExecutionError("Failed to submit name={self._name}")
+            # TODO: cancel or fail all jobs in the batch
+            logger.error("Failed to submit name=%s", self._name)
+            self._return_code = 1
+            self._is_complete = True
+            return Status.ERROR
 
         self._job_id = job_id
         event = StructuredLogEvent(
@@ -456,6 +483,7 @@ class AsyncHpcSubmitter(AsyncJobInterface):
         )
         log_event(event)
         logger.info("Assigned job_ID=%s name=%s", self._job_id, self._name)
+        return Status.GOOD
 
     def get_blocking_jobs(self):
         return set()
